#include	<avr/io.h>
//******************************************************************************************************************//
// Programme CAN0.c																									//
//																													//
// Ce programme effectue la conversion Analogique-Numrique de la tension Ve_CAN.									//
// La conversion est ralise  l'aide du convertisseur Analogique-Numrique intgr au C (Canal 0).				//
// La variable Ns0 correspond au rsultat de conversion de la tension applique sur le canal 0, exprim sur 10 bits.//					
//																													//
// Dans cette version, le CAN fonctionne en mode dclench.      													//
// Le programme effectue en permanence une conversion suivie d'une mmorisation du rsultat de conversion.			//
//																													//
// Le programme met  0 la broche Dem_conv_CAN2 lors de la demande de conversion, puis  1 lorsque la conversion 	//
// est termine. De cette faon, on peut mesurer le temps de conversion Tc du CAN intgr au C (dure au niveau  	//
// bas de Dem_conc_CAN2). Dem_conv_CAN2 constitue un simple drapeau (flag).											//
// On constate ainsi que la dure minimale sparant 2 conversions/mmorisatiions successives est de  111.9 s, 		//
// ce qui confre au dispositif [Acqurir + Traiter], une capacit de conversion de 8.9 k chantillons par seconde 	//
// (8.9 kSPS).	   																									//
//																													//
//******************************************************************************************************************//


//******************************************************************************************************************//
// Dfinition des masques associs aux bits assurant la gestion des diffrents convertisseurs						//
//******************************************************************************************************************//
#define	Dem_conv_CAN1	0x02
#define	Dem_lec_NS1		0x04
#define	Fin_conv_CAN1	0x08
#define	Dem_conv_CNA1	0x04
#define	Dem_conv_CAN2	0x20
#define	Dem_conv_CNA2	0x40
#define	Dem_conv_CAN3	0x20
#define	Fin_conv_CAN3	0x80

//******************************************************************************************************************//
// Dfinition des masques associs aux bits des registres de commande du CAN intgr au C							//
//******************************************************************************************************************//
#define	ADIF			0x10
#define ADSC			0x40



//******************************************************************************************************************//
// Dclaration des variables globales																				//
//******************************************************************************************************************//
struct	deux_octets
{
	unsigned char	bas;
	unsigned char	haut;
};

union	mot_de_16
{
	unsigned int	mot;
	struct	deux_octets	octet;
};

typedef union	mot_de_16	int16;

int16			Nb;
unsigned int	Ns0;				// Variable associe au rsultat de la conversion A-N de la tension Ve_CAN


//******************************************************************************************************************//
// Prototype des fonctions																							//
//******************************************************************************************************************//
void	init(void);
void	conv_CAN0(void);




//******************************************************************************************************************//
// Fonction principale																								//
//******************************************************************************************************************//
int main	(void)
{
	init();					// Excuter la fonction d'initialisation du C
	do						// 	FAIRE :
	{
		conv_CAN0();			// Excuter la fonction conv_CAN0() permettant de convertir Ve_CAN en un nb Ns0
	}
	while(1);				// EN PERMANENCE
}


//******************************************************************************************************************//
// Fonction init()																									//
//																													//
// Cette fonction configure l'ensemble des priphriques du C, y-compris ceux qui ne sont pas utiliss par			//
// l'application en cours.																							//
// Pour ce programme, on se limitera  prciser que la fonction init permet :										//
//	* de configurer la broches de port d'E/S associes au signal de commande Dem_conv_CAN2 en sortie				//
//	* de configurer le CAN intgr au C de faon  :																//
//		 slectionner le canal 0 (celui sur lequel la tension Ve_CAN est applique)								//
//		 slectionner une tension de rfrence (3.3V dans notre cas) externe										//
//		 slectionner une frquence de conversion interne de 125 kHz												//
//		 slectionner un fonctionnement en mode dclench															//
//******************************************************************************************************************//
void    init(void)
{
	DDRC=0xff;							// PortC (Ne1_0  Ne1_7) en sortie
	DDRG|=0x03;							// PG0 (Ne1_8) et PG1 (Ne1_9) en sortie
	PORTC=0x00;							// Mise  0 des 8 bits de poids faible de Ne1 (Ne1_0  Ne1_7)
	PORTG&=~0x03;						// Mise  0 des 2 bits de poids fort de Ne1 (Ne1_8 et Ne1_9)
	DDRA=0x00;							// PORTA (NS1_0  NS1_7) en entre
	DDRD&=~0x0c;						// PD2 (NS1_8) et PD3 (NS1_9) en entre
	DDRF|=(Dem_conv_CAN1+Dem_lec_NS1);	// Bits Dem_conv_CAN1 et Dem_lec_NS1 en sortie
	PORTF|=(Dem_conv_CAN1+Dem_lec_NS1);// Invalidation de demandes de conversion et de lecture du CAN1
	DDRF&=~Fin_conv_CAN1;				// PF3 (Fin_conv_CAN1) en entre
	DDRG|=Dem_conv_CNA1;				// PG2 (Dem_conv_CNA1) en sortie
	PORTG|=Dem_conv_CNA1;				// Invalidation de demande de conversion du CNA1
	DDRE|=Dem_conv_CAN2;				// PE5 (Dem_conv_CAN2) en entre
	PORTE|=Dem_conv_CAN2;				// Invalidation de demande de conversion du CAN2
	DDRE|=Dem_conv_CNA2;				// PE6 (Dem_conv_CNA2) en entre
	PORTE|=Dem_conv_CNA2;				// Invalidation de demande de conversion du CNA2
	DDRD|=Dem_conv_CAN3;				// PD5 (Dem_conv_CAN3) en entre
	PORTD|=Dem_conv_CAN3;				// Invalidation de demande de conversion du CAN3
	DDRD&=~Fin_conv_CAN3;				// PD7 (Fin_conv_CAN3) en entre
	ADMUX=0x00;							// Tension de rfrence du CAN_C externe et validation du canal 0
	ADCSRA=0xc6;						// Validation du CAN avec prdivision de frquence par 64, ce qui correspond 
										// une horloge CAN interne de frquence fQuartz/64 = 125kHz, et fonctionnement
										// en mode dclench
	while(!(ADCSRA&ADIF));				// Attendre que la 1re conversion d'initialisation soit termine
	ADCSRA|=ADIF;						// Faire une RAZ du bit indicateur de fin de conversion
}


//******************************************************************************************************************//
// Fonction conv_CAN0()																								//
//																													//
// Cette fonction effectue la conversion Analogique-Numrique de la tension Ve_CAN et sauvegarde le rsultat de		//
// conversion dans la variable Ns0.																					//
//******************************************************************************************************************//
void	conv_CAN0(void)
{
	ADCSRA|=ADSC;				// Demander la conversion
	PORTE&=~Dem_conv_CAN2;		// Mettre le flag de dbut de conversion  0
	while(!(ADCSRA&ADIF));		// Attendre la fin de la conversion
	PORTE|=Dem_conv_CAN2;		// Mettre le flag de fin de conversion  1
	ADCSRA|=ADIF;				// Faire une RAZ du bit indicateur de fin de conversion
	Nb.octet.bas=ADCL;			// Mmoriser les 8 bits de poids faible dU rsultat de conversion dans Nb.octet.bas
	Nb.octet.haut=ADCH;			// Mmoriser les 2 bits de poids fort du rsultat de conversion dans Nb.octet.haut
	Ns0=Nb.mot;					// Initialiser la variable NS0 avec le rsultat de conversion
}

