#include	<avr/io.h>
#include	<util/delay.h>
//******************************************************************************************************************//
// Programme CAN1.c																									//
//																													//
// Ce programme effectue la conversion Analogique-Numrique de la tension Ve_CAN.									//
// La conversion est ralise  l'aide du convertisseur Analogique-Numrique CAN1.									//
// La variable Ns1 correspond au rsultat de conversion, exprim sur 10 bits.										//
// 																													//
// Le programme effectue en permanence une conversion suivie d'une mmorisation du rsultat de conversion.			//
//																													//
// 0n constate que la dure minimale sparant 2 conversions/mmorisatiions successives est de 7.4 s, ce qui		//
// confre au dispositif [Acqurir + Traiter], une capacit de conversion de 135.6 k chantillons par seconde     	//
// (135.6 kSPS).																										//
//																													//
// La dure d'excution de la fonction conv_CAN1, constitue des 3 phases successives :								//
//		* laboration de l'impulsion de demande de conversion														//
//		* test d'attente de fin de conversion																		//
//		* lecture du rsultat de conversion puis mmorisation dans la variable Ns1									//
// est gale  5.5 s.																								//
//																													//
// Par ailleurs, on remarque que le signal BUSY reste au niveau haut pendant 500 ns, ce qui correspond au temps de  //
// conversion Tc interne du CAN1, gal, d'aprs la documentation du CAN1  12/Fclk, soit 12/24MHz = 500 ns.			//
//																													//
//******************************************************************************************************************//

//******************************************************************************************************************//
// Dfinition des masques associs aux bits assurant la gestion des diffrents convertisseurs						//
//******************************************************************************************************************//
#define	Dem_conv_CAN1	0x02
#define	Dem_lec_NS1		0x04
#define	Fin_conv_CAN1	0x08
#define	Dem_conv_CNA1	0x04
#define	Dem_conv_CAN2	0x20
#define	Dem_conv_CNA2	0x40
#define	Dem_conv_CAN3	0x20
#define	Fin_conv_CAN3	0x80

//******************************************************************************************************************//
// Dclaration des variables globales																				//
//******************************************************************************************************************//
struct	deux_octets
	{
	unsigned char	bas;
	unsigned char	haut;
	};

union	mot_de_16
	{
	unsigned int	mot;
	struct	deux_octets	octet;
	};

typedef union	mot_de_16	int16;

int16			Nb;
unsigned int	Ns1;				// Variable associe au rsultat de la conversion A-N de la tension Ve_CAN



//******************************************************************************************************************//
// Prototype des fonctions																							//
//******************************************************************************************************************//
void	init(void);
void	conv_CAN1(void);



//******************************************************************************************************************//
// Fonction principale																								//
//******************************************************************************************************************//
int main	(void)
{
	init();					// Excuter la fonction d'initialisation du C
	do						// 	FAIRE :
	{
		conv_CAN1();			// Excuter la fonction conv_CAN1 permettant de convertir Ve_CAN en un nombre Ns1
	}
	while(1);				// EN PERMANENCE
}


//******************************************************************************************************************//
// Fonction init()																									//
//																													//
// Cette fonction configure l'ensemble des priphriques du C, y-compris ceux qui ne sont pas utiliss par			//
// l'application en cours.																							//
// Pour ce programme, on se limitera  prciser que la fonction init permet :										//
//	* de configurer les broches de port d'E/S associes aux signaux de cde Dem_conv_CAN1 et Dem_lec_Ns1 en sorties	//
//	* de configurer la broche de port d'E/S associe au signal de commande Fin_conv_CAN1 en entre					//
//	* de configurer les 10 broches de port d'E/S connectes aux 10 sorties du CAN1 en entres						//
//******************************************************************************************************************//
void    init(void)
{
	DDRC=0xff;							// PortC (Ne1_0  Ne1_7) en sortie
	DDRG|=0x03;							// PG0 (Ne1_8) et PG1 (Ne1_9) en sortie
	PORTC=0x00;							// Mise  0 des 8 bits de poids faible de Ne1 (Ne1_0  Ne1_7)
	PORTG&=~0x03;						// Mise  0 des 2 bits de poids fort de Ne1 (Ne1_8 et Ne1_9)
	DDRA=0x00;							// PORTA (NS1_0  NS1_7) en entre
	DDRD&=~0x0c;						// PD2 (NS1_8) et PD3 (NS1_9) en entre
	DDRF|=(Dem_conv_CAN1+Dem_lec_NS1);	// Bits Dem_conv_CAN1 et Dem_lec_NS1 en sortie
	PORTF|=(Dem_conv_CAN1+Dem_lec_NS1);// Invalider toute demande de conversion et de lecture du CAN1
	DDRF&=~Fin_conv_CAN1;				// PF3 (Fin_conv_CAN1) en entre
	DDRG|=Dem_conv_CNA1;				// PG2 (Dem_conv_CNA1) en sortie
	PORTG|=Dem_conv_CNA1;				// Invalider toute demande de conversion du CNA1
	DDRE|=Dem_conv_CAN2;				// PE5 (Dem_conv_CAN2) en entre
	PORTE|=Dem_conv_CAN2;				// Invalider toute demande de conversion du CAN2
	DDRE|=Dem_conv_CNA2;				// PE6 (Dem_conv_CNA2) en entre
	PORTE|=Dem_conv_CNA2;				// Invalider toute demande de conversion du CNA2
	DDRD|=Dem_conv_CAN3;				// PD5 (Dem_conv_CAN3) en entre
	PORTD|=Dem_conv_CAN3;				// Invalider toute demande de conversion du CAN3
	DDRD&=~Fin_conv_CAN3;				// PD7 (Fin_conv_CAN3) en entre
	Nb.octet.haut=0x00;
	PORTD&=~0x0c;
	PORTA=0x00;
}


//******************************************************************************************************************//
// Fonction conv_CAN1()																								//
//																													//
// Cette fonction effectue la conversion Analogique-Numrique de la tension Ve_CAN et sauvegarde le rsultat de		//
// conversion dans la variable Ns1.																					//
//******************************************************************************************************************//
void	conv_CAN1(void)
{
	PORTF&=~Dem_conv_CAN1;				// Elaborer le front descendant de l'impulsion de demande de conversion		
	PORTF|=Dem_conv_CAN1;				// Elaborer le front montant de l'impulsion de demande de conversion
	do {}								// Tester le bit indicateur de fin de conversion et attendre que la conversion
	while((PINF&Fin_conv_CAN1)!=0);	// soit termine
	PORTF&=~Dem_lec_NS1;				// Elaborer le signal demandant au CAN de dlivrer son rsultat de conversion
	_delay_us(1);						// Attendre 1 s (tempo. non prdictible, mais ncessaire exprimentalement) 
	Nb.octet.bas=PINA;					// Rcuprer les 8 bits de poids faible du rsultat de conversion
	Nb.octet.haut=((PIND&0x0c)>>2);	// Rcuprer les 2 bits de poids fort du rsultat de conversion
	Ns1=Nb.mot;							// Initialiser la variable Ns1 avec le rsultat de conversion
	PORTF|=Dem_lec_NS1;					// Elaborer le signal demandant au CAN de cesser de dlivrer son rsultat
}
