#include	<avr/io.h>
//******************************************************************************************************************//
// Programme CAN2.c																									//
//																													//
// Ce programme effectue la conversion Analogique-Numrique de la tension Ve_CAN.									//
// La conversion est ralise  l'aide du convertisseur Analogique-Numrique CAN2.									//
// La variable Ns2 correspond au rsultat de conversion, exprim sur 10 bits.										//
// 																													//
// Le programme effectue en permanence une conversion suivie d'une mmorisation du rsultat de conversion.			//
//																						 							//
// On constate que la dure minimale sparant 2 conversions/mmorisations successives est de 12.1 s, ce qui		//
// confre au dispositif [Acqurir + Traiter], une capacit de conversion de 82.5 k chantillons par seconde     	//
// (82.5 kSPS).																										//
// 																													//
//******************************************************************************************************************//

//******************************************************************************************************************//
// Dfinition des masques associs aux bits assurant la gestion des diffrents convertisseurs						//
//******************************************************************************************************************//
#define	Dem_conv_CAN1	0x02
#define	Dem_lec_NS1		0x04
#define	Fin_conv_CAN1	0x08
#define	Dem_conv_CNA1	0x04
#define	Dem_conv_CAN2	0x20
#define	Dem_conv_CNA2	0x40
#define	Dem_conv_CAN3	0x20
#define	Fin_conv_CAN3	0x80




//******************************************************************************************************************//
// Dfinition des masques associs aux bits des registres de commande de l'interface srie synchrone SPI			//
//******************************************************************************************************************//
#define	SS				0x01
#define	SCK				0x02
#define	MOSI			0x04
#define	MISO			0x08
#define	SPI2X			0x01
#define	SPIF			0x80


//******************************************************************************************************************//
// Dclaration des variables globales																				//
//******************************************************************************************************************//
struct	deux_octets
{
	unsigned char	bas;
	unsigned char	haut;
};

union	mot_de_16
{
	unsigned int	mot;
	struct	deux_octets	octet;
};

typedef union	mot_de_16	int16;

int16			Nb;
unsigned int	Ns2;				// Variable associe au rsultat de la conversion A-N de la tension Ve_CAN



//******************************************************************************************************************//
// Prototype des fonctions																							//
//******************************************************************************************************************//
void	init(void);
void	conv_CAN2(void);


//******************************************************************************************************************//
// Fonction principale																								//
//******************************************************************************************************************//
int main	(void)
{
	init();					// Excuter la fonction d'initialisation du C
	do						// 	FAIRE :
	{
			conv_CAN2();			// Excuter la fonction conv_CAN2 permettant de convertir Ve_CAN en un nombre Ns2
	}
	while(1);				// EN PERMANENCE
}


//******************************************************************************************************************//
// Fonction init()																									//
//																													//
// Cette fonction configure l'ensemble des priphriques du C, y-compris ceux qui ne sont pas utiliss par			//
// l'application en cours.																							//
// Pour ce programme, on se limitera  prciser que la fonction init permet :										//
//	* de configurer la broche de port d'E/S associe au signal de commande Dem_conv_CAN2 en sortie					//
//	* de configurer l'interface SPI de faon  pouvoir piloter le convertisseur A-N CAN2, c'est  dire :			//
//		 mode Matre																								//
//		 transmission/rception bit de poids fort en 1re position (MSB first)										//
//		 horloge inactive au niveau haut																			//
//		 changement de niveau logique des bits de donnes en synchronisme avec les fronts descendants d'horloge	//
//		 frquence d'horloge gale  la frquence du quartz du C divise par 2, soit 4MHz							//
//******************************************************************************************************************//
void    init(void)
{
	DDRC=0xff;							// PortC (Ne1_0  Ne1_7) en sortie
	DDRG|=0x03;							// PG0 (Ne1_8) et PG1 (Ne1_9) en sortie
	PORTC=0x00;							// Mise  0 des 8 bits de poids faible de Ne1 (Ne1_0  Ne1_7)
	PORTG&=~0x03;						// Mise  0 des 2 bits de poids fort de Ne1 (Ne1_8 et Ne1_9)
	DDRA=0x00;							// PORTA (NS1_0  NS1_7) en entre
	DDRD&=~0x0c;						// PD2 (NS1_8) et PD3 (NS1_9) en entre
	DDRF|=(Dem_conv_CAN1+Dem_lec_NS1);	// Bits Dem_conv_CAN1 et Dem_lec_NS1 en sortie
	PORTF|=(Dem_conv_CAN1+Dem_lec_NS1);// Invalidation de demandes de conversion et de lecture du CAN1
	DDRF&=~Fin_conv_CAN1;				// PF3 (Fin_conv_CAN1) en entre
	DDRG|=Dem_conv_CNA1;				// PG2 (Dem_conv_CNA1) en sortie
	PORTG|=Dem_conv_CNA1;				// Invalidation de demande de conversion du CNA1
	DDRE|=Dem_conv_CAN2;				// PE5 (Dem_conv_CAN2) en entre
	PORTE|=Dem_conv_CAN2;				// Invalidation de demande de conversion du CAN2
	DDRE|=Dem_conv_CNA2;				// PE6 (Dem_conv_CNA2) en entre
	PORTE|=Dem_conv_CNA2;				// Invalidation de demande de conversion du CNA2
	DDRD|=Dem_conv_CAN3;				// PD5 (Dem_conv_CAN3) en entre
	PORTD|=Dem_conv_CAN3;				// Invalidation de demande de conversion du CAN3
	DDRD&=~Fin_conv_CAN3;				// PD7 (Fin_conv_CAN3) en entre
	DDRB|=(SCK+MOSI+SS);				// Broches /SS, SCK et MOSI en sortie
	DDRB&=~MISO;						// Broche MISO en entre
	SPCR=0x5c;							// Validation de la SPI en matre avec transmission MSB first, et CPOL=CPHA=1
	SPSR|=SPI2X;						// Horloge SCK de frquence fQuartz/2 = 4MHz
}



//******************************************************************************************************************//
// Fonction conv_CAN2()																								//
// 																													//
// Cette fonction effectue la conversion Analogique-Numrique de la tension Ve_CAN et sauvegarde le rsultat de		//
// conversion dans la variable Ns2.																					//
//******************************************************************************************************************//
void	conv_CAN2(void)
{
	PORTE&=~Dem_conv_CAN2;		// Valider le convertisseur A-N  interface srie SPI CAN2
	SPDR=0xAA;					// Donner l'ordre de transmission de la donne factice 0xAA
	while(!(SPSR&SPIF));		// Attendre que la donne 0xAA soit totalement transmise
	Nb.octet.haut=SPDR;			// Rcuprer la 1re moiti du rsultat de conversion transmis par le CAN2
	SPDR=0x55;					// Donner l'ordre de transmission de la donne factice 0x55
	while(!(SPSR&SPIF));		// Attendre que la donne 0x55 soit totalement transmise
	Nb.octet.bas=SPDR;			// Rcuprer la 2me moiti du rsultat de conversion transmis par le CAN2
	PORTE|=Dem_conv_CAN2;		// Invalider le convertissuer A-N  interface srie SPI CAN2
	Ns2=Nb.mot>>5;				// Concatnr les deux "moitis" de rsultats de conversion, puis initialiser la
								// variable Ns2 avec le mot de 16 bits issu de la concatnation divis par 32
}
