#include	<avr/io.h>
#include	<util/delay.h>
//******************************************************************************************************************//
// Programme CAN3.c																									//
//																													//
// Ce programme effectue la conversion Analogique-Numrique de la tension Ve_CAN.									//
// La conversion est ralise  l'aide du convertisseur Analogique-Numrique CAN3 configur en mode 1.				//
// La variable Ns3 correspond au rsultat de conversion, exprim sur 10 bits.										//
//																													//
// Le programme effectue en permanence une conversion suivie d'une mmorisation du rsultat de conversion.			//
//																													//
// 0n constate que la dure minimale sparant 2 conversions/mmorisatiions successives est de 83.7 s, ce qui		//
// confre au dispositif [Acqurir + Traiter], une capacit de conversion de 11.9 k chantillons par seconde     	//
// (11.9 kSPS).																										//
// 																													//
// La dure de conversion proprement dite est de 2 s (valeur annonce par le constructeur mais non mesurable).		//
//																													//
//******************************************************************************************************************//

//******************************************************************************************************************//
// Dfinition des masques associs aux bits des registres de commande de l'interface srie synchrone I2C			//
//******************************************************************************************************************//
#define	Dem_conv_CAN1	0x02
#define	Dem_lec_NS1		0x04
#define	Fin_conv_CAN1	0x08
#define	Dem_conv_CNA1	0x04
#define	Dem_conv_CAN2	0x20
#define	Dem_conv_CNA2	0x40
#define	Dem_conv_CAN3	0x20
#define	Fin_conv_CAN3	0x80

//******************************************************************************************************************//
// Dfinition des masques associs aux bits des registres de commande de l'interface srie synchrone I2C			//
//******************************************************************************************************************//
#define	TWINT			0x80
#define	TWSTA			0x20
#define	TWSTO			0x10
#define	TWEN			0X04
#define LECTURE			1
#define ECRITURE		0
#define	Ad_CAN3			0x22		// Adresse I2C du CAN3
#define	Ad_Reg_Config	0x02
#define	Ad_Reg_Result	0x00
#define	Ad_Canal_1		0x10
#define	T_Attente		2
#define	T_PowerUp		1
#define	T_Convert		2




//******************************************************************************************************************//
// Dclaration des variables globales																				//
//******************************************************************************************************************//
struct	deux_octets
	{
	unsigned char	bas;
	unsigned char	haut;
	};

union	mot_de_16
	{
	unsigned int	mot;
	struct	deux_octets	octet;
	};

typedef union	mot_de_16	int16;

int16			Nb;
unsigned int	Ns3;				// Variable associe au rsultat de la conversion A-N de la tension Ve_CAN


//******************************************************************************************************************//
// Prototype des fonctions																							//
//******************************************************************************************************************//
void	init(void);
void	conv_CAN3(void);


//******************************************************************************************************************//
// Fonction principale																								//
//******************************************************************************************************************//
int main	(void)
{
	init();					// Excuter la fonction d'initialisation du C
	do						// 	FAIRE :
	{
			conv_CAN3();			// Excuter la fonction conv_CAN3 permettant de convertir Ve_CAN en un nombre Ns3
	}
	while(1);				// EN PERMANENCE
}


//******************************************************************************************************************//
// Fonction init()																									//
//																													//
// Cette fonction configure l'ensemble des priphriques du C, y-compris ceux qui ne sont pas utiliss par			//
// l'application en cours.																							//
// Pour ce programme, on se limitera  prciser que la fonction init permet :										//
//	* de configurer la broche de port d'E/S associe au signal de commande Dem_conv_CAN3 en sortie					//
//	* de configurer l'interface SPI de faon  pouvoir piloter le convertisseur A-N CAN3							//
//******************************************************************************************************************//
void    init(void)
{
	DDRC=0xff;							// PortC (Ne1_0  Ne1_7) en sortie
	DDRG|=0x03;							// PG0 (Ne1_8) et PG1 (Ne1_9) en sortie
	PORTC=0x00;							// Mise  0 des 8 bits de poids faible de Ne1 (Ne1_0  Ne1_7)
	PORTG&=~0x03;						// Mise  0 des 2 bits de poids fort de Ne1 (Ne1_8 et Ne1_9)
	DDRA=0x00;							// PORTA (NS1_0  NS1_7) en entre
	DDRD&=~0x0c;						// PD2 (NS1_8) et PD3 (NS1_9) en entre
	DDRF|=(Dem_conv_CAN1+Dem_lec_NS1);	// Bits Dem_conv_CAN1 et Dem_lec_NS1 en sortie
	PORTF|=(Dem_conv_CAN1+Dem_lec_NS1);// Invalidation de demandes de conversion et de lecture du CAN1
	DDRF&=~Fin_conv_CAN1;				// PF3 (Fin_conv_CAN1) en entre
	DDRG|=Dem_conv_CNA1;				// PG2 (Dem_conv_CNA1) en sortie
	PORTG|=Dem_conv_CNA1;				// Invalidation de demande de conversion du CNA1
	DDRE|=Dem_conv_CAN2;				// PE5 (Dem_conv_CAN2) en sortie
	PORTE|=Dem_conv_CAN2;				// Invalidation de demande de conversion du CAN2
	DDRE|=Dem_conv_CNA2;				// PE6 (Dem_conv_CNA2) en sortie
	PORTE|=Dem_conv_CNA2;				// Invalidation de demande de conversion du CNA2
	DDRD|=Dem_conv_CAN3;				// PD5 (Dem_conv_CAN3) en sortiee
	PORTD&=~Dem_conv_CAN3;				// Invalidation de demande de conversion du CAN3
	DDRD&=~Fin_conv_CAN3;				// PD7 (Fin_conv_CAN3) en entre
	TWBR=0x02;							// FSCL = 400 kHz
	TWCR|=(TWINT+TWSTA+TWEN);			// Envoi de la commande de START
	while(!(TWCR&TWINT));				// Attendre fin de transmission de la commande
	TWDR=((Ad_CAN3<<1)+ECRITURE);		// Prparation de la commande d'accs au CAN3 en criture
	TWCR=(TWCR|0xA4)&0xCD;				// Envoi de la commande d'accs au CAN3 en criture
	while(!(TWCR&TWINT));				// Attendre fin de transmission de la commande
	TWDR=Ad_Reg_Config;					// Prparation de la slection du registre de configuration
	TWCR=(TWCR|0xA4)&0xCD;				// Envoi de la commande de slection du registre de configuration
	while(!(TWCR&TWINT));				// Attendre fin de transmission de la commande
	TWDR=Ad_Canal_1;					// Prparation de la slection du canal de conversion n1
	TWCR=(TWCR|0xA4)&0xCD;				// Envoi de la commande de slection du canal de conversion n1
	while(!(TWCR&TWINT));				// Attendre fin de transmission de la commande
	TWCR=(TWCR|0x94)&0xDD;				// Envoi de la commande de STOP
	TWCR=(TWCR|0xA4)&0xED;				// Envoi de la commande de START
	while(!(TWCR&TWINT));				// Attendre fin de transmission de la commande
	TWDR=((Ad_CAN3<<1)+ECRITURE);		// Prparation de la commande d'accs au CAN3 en criture
	TWCR=(TWCR|0xA4)&0xCD;				// Envoi de la commande d'accs au CAN3 en criture
	while(!(TWCR&TWINT));				// Attendre fin de transmission de la commande
	TWDR=Ad_Reg_Result;					// Prparation de la slection du registre rsultat de conversiopn
	TWCR=(TWCR|0xA4)&0xCD;				// Envoi de la commande de slection du registre rsultat de conversion
	while(!(TWCR&TWINT));				// Attendre fin de transmission de la commande
	TWCR=(TWCR|0x94)&0xDD;				// Envoi de la commande de STOP
}



//******************************************************************************************************************//
// Fonction conv_CAN3()																								//
// 																													//
// Cette fonction effectue la conversion Analogique-Numrique de la tension Ve_CAN et sauvegarde le rsultat de		//
// conversion dans la variable Ns3.																					//
//******************************************************************************************************************//
void	conv_CAN3(void)
{
	PORTD|=Dem_conv_CAN3;			// Mise au niveau haut de l'impulsion de demande de conversion
	_delay_us(T_PowerUp);			// Maintien du niveau haut de l'impulsion pendant 1s
	PORTD&=~Dem_conv_CAN3;			// Mise au niveau bas de l'impulsion de demande de conversion
	_delay_us(T_Convert);			// Attendre 2s que la conversion A-N soit termine
	TWCR=(TWCR|0xA4)&0xED;			// Envoi de la commande de START
	while(!(TWCR&TWINT));			// Attendre fin de transmission de la commande
	TWDR=((Ad_CAN3<<1)+LECTURE);	// Prparation de la commande d'accs au CAN3 en lecture
	TWCR=(TWCR|0xA4)&0xCD;			// Envoi de la commande d'accs au CAN3 en lecture
	while(!(TWCR&TWINT));			// Attendre fin de transmission de la commande
	TWCR=(TWCR|0xE4)&0xCD;			// Envoi de la commande de lecture de l'octet de poids fort
	while(!(TWCR&TWINT));			// Attendre fin de rception de l'octet de poids fort
	Nb.octet.haut=TWDR;				// Rcuprer la 1re moiti du rsultat de conversion transmis par le CAN3
	TWCR=(TWCR|0xA4)&0x8D;			// Envoi de la commande de lecture de l'octet de poids faible
	while(!(TWCR&TWINT));			// Attendre fin de rception de l'octet de poids faible
	Nb.octet.bas=TWDR;				// Rcuprer la 2me moiti du rsultat de conversion transmis par le CAN3
	TWCR=(TWCR|0x94)&0xDD;			// Envoi de la commande de STOP	
	Ns3=((Nb.mot)&0x0fff)>>2;		// Concatnr les deux "moitis" de rsultat de conversion, puis initialiser la
									// variable Ns3 avec le mot de 16 bits issu de la concatnation divis par 4
}



